import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import {
    types,
    Tab,
    Dialog,
    Button,
    Search,
    Select,
    TagSelector,
    FileSelect,
    KZUIComponent,
} from 'kui2';
import PicLib from './components/piclib';
import Progressbar from '../progressbar/index';
import {
    optionsFetch,
    galleryFetch,
    selectChange,
    tagChange,
    searchGallery,
    PicFetchNext,
    GalleryFetchNext,
    refreshPic,
    selectPic,
    displayUpload,
    dismissUpload,
    uploadIndex,
    confirmPicSelect,
} from './actions';
import {
    showAlert,
} from '../alert/actions';
import './style.less';
import { picListType } from './types';

class PicSelector extends KZUIComponent {
    constructor(props) {
        super(props);
        this.autoBind('handleConfirm', 'handleCancel', 'handleSelectChange', 'handleTagChange',
            'handleSearch', 'mypicFetchNext', 'galleryFetchNext', 'handleFiles', 'handleFileError', 'handleSelectPic');
    }

    componentWillMount() {
        const { ds, siteId } = this.context;
        this.props.dispatch(optionsFetch(ds, siteId));
        this.props.dispatch(galleryFetch(ds, {
            page: 1,
            per_page: 15,
            cate: this.props.curTag,
            query: this.props.query,
            site_id: siteId,
        }));
    }

    handleConfirm() {
        this.props.dispatch(confirmPicSelect());
        if (this.props.selected.length > 0 && this.props.onConfirm) {
            this.props.onConfirm(this.props.selected);
        }
    }

    handleCancel() {
        if (this.props.onCancel) {
            this.props.onCancel();
        }
    }

    handleSelectChange(data) {
        const { ds, siteId } = this.context;
        this.props.dispatch(selectChange(ds, siteId, data.value));
    }

    handleTagChange(data) {
        const { ds, siteId } = this.context;
        this.props.dispatch(tagChange(ds, siteId, data[0].name));
    }

    handleSearch(data) {
        const { ds, siteId } = this.context;
        this.props.dispatch(searchGallery(ds, siteId, data.value));
    }

    mypicFetchNext() {
        if (this.props.picMore) {
            const { ds, siteId } = this.context;
            this.props.dispatch(PicFetchNext(ds, siteId));
        }
    }

    galleryFetchNext() {
        if (this.props.galleryMore) {
            const { ds, siteId } = this.context;
            this.props.dispatch(GalleryFetchNext(ds, siteId));
        }
    }

    handleFiles(data) {
        const files = [].slice.apply(data);
        const { ds, siteId } = this.context;
        if (files.length === 0) {
            return;
        }
        this.props.dispatch(displayUpload(files.length));

        let index = 0;
        let err = 0;
        const name = `upload_${new Date().getTime()}`;
        const complete = () => {
            this.props.dispatch(uploadIndex(index));
            if (index === files.length) {
                if (err > 0) {
                    this.props.dispatch(showAlert(`共上传${index}张，失败${err}`));
                } else {
                    this.props.dispatch(showAlert(`已完成，共上传${index}张`));
                }
                this.props.dispatch(refreshPic(ds, siteId));
                this.props.dispatch(dismissUpload());
            }
        };

        files.forEach((file) => {
            const fd = new FormData();
            fd.append('apic', file);
            fd.append('site_id', siteId);
            fd.append('tag_name', this.props.curSelect);
            fd.append('callback', `SOHUZ.${name}`);
            ds.resolve('v1/pic-upload', {
                body: fd,
            }).then((ret) => {
                index += 1;
                if (ret.status !== 200) {
                    err += 1;
                }
                complete();
            }).catch(() => {
                err += 1;
                index += 1;
                complete();
            });
        });
    }

    handleFileError(error) {
        this.props.dispatch(showAlert(error.toString()));
    }

    handleSelectPic(pic) {
        this.props.dispatch(selectPic(pic));
    }

    render() {
        const { siteId } = this.context;
        const tabTitles = [
            ['我的图片', 'js-tab-gallery'],
            ['素材库', 'js-tab-piclib'],
        ];
        const spans = [
            '全部', '风景', '静物', '美女', '美男', '卡通', '心情', '汽车',
            '影视', '游戏', '体育', '品牌', '其他',
        ];
        const width = `${(this.props.uploadIndex / this.props.uploadTotal) * 100}%`;
        let lib = (<div className="nocontent">暂无图片</div>);
        if (this.props.picList.length > 0) {
            lib = (
                <PicLib
                    key={this.props.curSelect}
                    list={this.props.picList}
                    loading
                    onScroll={this.mypicFetchNext}
                    onClick={this.handleSelectPic}
                />
            );
        }
        if (this.props.uploading) {
            lib = (
                <div className="uploading">
                    <p>正在上传...</p>
                    <span>
                        完成
                        <span className="uploaded">{this.props.uploadIndex}</span>
                        张，共
                        <span className="total">{this.props.uploadTotal}</span>
                        张
                    </span>
                    <Progressbar width={width} />
                </div>
            );
        }

        return (
            <Dialog className="pic-selector" hide={this.props.hide}>
                <Tab tabTitles={tabTitles}>
                    <div className="my-pic">
                        <Select options={this.props.options} onChange={this.handleSelectChange} />
                        <a className="manage-pic" href={`/pic/?site_id=${siteId}`}>管理图片</a>
                        <span className="note">提示</span>
                        <span>建议图片宽度640像素以上，支持多图上传，图片大小不能超过4M，支持jpg，jpeg，png，gif等格式</span>
                        <FileSelect
                            iconClass="kz-e-system_plugin-pic-min"
                            multiple
                            maxFileCount={10}
                            onSelect={this.handleFiles}
                            onError={this.handleFileError}
                        >
                            上传图片
                        </FileSelect>
                        {lib}
                    </div>
                    <div className="gallery">
                        <TagSelector
                            tags={spans}
                            default={spans.indexOf(this.props.curTag)}
                            onChange={this.handleTagChange}
                        />
                        <Search
                            placeholder="搜索图片"
                            value={this.props.query}
                            onSearch={this.handleSearch}
                        />
                        <PicLib
                            key={this.props.curTag}
                            list={this.props.galleryList}
                            loading
                            onScroll={this.galleryFetchNext}
                            onClick={this.handleSelectPic}
                        />
                    </div>
                </Tab>
                <div className={this.classname('btn-con')}>
                    <label className="tip">使用须知</label>
                    <span className="tip-content">
                        素材库部分资源来自网络，版权归原作者所有。如有问题敬请告知，我们会立即处理。联系电话400-6819-018
                    </span>
                    <Button
                        className="button cancel-pic"
                        data-dismiss="modal"
                        onClick={this.handleCancel}
                    >
                        取消
                    </Button>
                    <Button
                        className="confirm confirm-pic"
                        data-action="confirm"
                        onClick={this.handleConfirm}
                    >
                        确定
                    </Button>
                </div>
            </Dialog>
        );
    }
}

PicSelector.contextTypes = {
    ds: PropTypes.object,
    siteId: PropTypes.number,
};

PicSelector.propTypes = {
    hide: PropTypes.bool,
    query: PropTypes.string,
    curTag: PropTypes.string,
    curSelect: PropTypes.string,
    alter: PropTypes.bool,
    alterMsg: PropTypes.string,
    selected: picListType,
    options: types.optionListType,
    picList: picListType,
    galleryList: picListType,
    picPage: PropTypes.number,
    picMore: PropTypes.bool,
    galleryPage: PropTypes.number,
    galleryMore: PropTypes.bool,
    uploading: PropTypes.bool,
    uploadIndex: PropTypes.number,
    uploadTotal: PropTypes.number,
    onConfirm: PropTypes.func,
    onCancel: PropTypes.func,
};

PicSelector.defaultProps = {
    hide: true,
    query: '',
    curTag: '全部',
    curSelect: '',
    alter: false,
    alterMsg: '',
    selected: [],
    options: [],
    picList: [],
    galleryList: [],
    picPage: 1,
    picMore: true,
    galleryPage: 1,
    galleryMore: true,
    uploading: false,
    uploadIndex: 0,
    uploadTotal: 0,
    onConfirm: null,
    onCancel: null,
};

export default connect(
    state => (state.picSelector),
)(PicSelector);
